..\..\MultiRecorder.exe --run sample6.xml --log log_sample6.txt
..\..\MultiRecorder.exe --export-tiffs sample6-recorded.xirecsession --export-tiffs-folder sample6-exported --log log_sample6export.txt
