

#pragma once
// This source file contains camera model enumerators generated by sensors_enum.xslt

#ifdef __cplusplus
extern "C" {
#endif

typedef enum
{
	SENSOR_MODEL_ID_UNKNOWN			=0,
  
	SENSOR_MODEL_ID_Sony_ICX282AQ = 1,
	
	SENSOR_MODEL_ID_Sony_ICX252AQ = 2,
	
	SENSOR_MODEL_ID_Sony_ICX285AQ = 3,
	
	SENSOR_MODEL_ID_Sony_ICX285AL = 4,
	
	SENSOR_MODEL_ID_Sony_ICX456AQ = 5,
	
	SENSOR_MODEL_ID_Sony_ICX274AQ = 6,
	
	SENSOR_MODEL_ID_Sony_ICX274AL = 7,
	
	SENSOR_MODEL_ID_Sony_ICX413AQ = 8,
	
	SENSOR_MODEL_ID_OnSemi_KAI4021 = 9,
	
	SENSOR_MODEL_ID_OnSemi_KAI11002 = 10,
	
	SENSOR_MODEL_ID_OnSemi_KAI16000 = 11,
	
	SENSOR_MODEL_ID_Aptina_MT9P031 = 12,
	
	SENSOR_MODEL_ID_Aptina_MT9P034 = 13,
	
	SENSOR_MODEL_ID_e2v_EV76C560 = 14,
	
	SENSOR_MODEL_ID_CMOSIS_CMV300 = 16,
	
	SENSOR_MODEL_ID_CMOSIS_CMV2000 = 17,
	
	SENSOR_MODEL_ID_CMOSIS_CMV4000 = 18,
	
	SENSOR_MODEL_ID_Sony_ICX655AQA = 19,
	
	SENSOR_MODEL_ID_e2v_EV76C661 = 21,
	
	SENSOR_MODEL_ID_OnSemi_VITA1300 = 22,
	
	SENSOR_MODEL_ID_e2v_EV76C570 = 23,
	
	SENSOR_MODEL_ID_OnSemi_KAI4022 = 25,
	
	SENSOR_MODEL_ID_Aptina_MT9J003 = 26,
	
	SENSOR_MODEL_ID_Sony_ICX674AL = 27,
	
	SENSOR_MODEL_ID_Sony_ICX694AL = 28,
	
	SENSOR_MODEL_ID_Sony_ICX814AL = 29,
	
	SENSOR_MODEL_ID_Sony_ICX834AL = 30,
	
	SENSOR_MODEL_ID_CMOSIS_CMV20000 = 31,
	
	SENSOR_MODEL_ID_CMOSIS_CMV12000 = 32,
	
	SENSOR_MODEL_ID_Sony_IMX174 = 33,
	
	SENSOR_MODEL_ID_Sony_ICX674AQ = 34,
	
	SENSOR_MODEL_ID_Sony_ICX694AQ = 35,
	
	SENSOR_MODEL_ID_Sony_ICX814AQ = 36,
	
	SENSOR_MODEL_ID_Sony_ICX834AQ = 37,
	
	SENSOR_MODEL_ID_Luxima_LUX330 = 38,
	
	SENSOR_MODEL_ID_OnSemi_PYTHON1300 = 39,
	
	SENSOR_MODEL_ID_Sony_IMX250 = 40,
	
	SENSOR_MODEL_ID_Sony_IMX252 = 41,
	
	SENSOR_MODEL_ID_Sony_IMX253 = 42,
	
	SENSOR_MODEL_ID_Sony_IMX255 = 43,
	
	SENSOR_MODEL_ID_CMOSIS_CMV50000 = 44,
	
	SENSOR_MODEL_ID_GPixel_GSENSE2020e = 45,
	
	SENSOR_MODEL_ID_LUXIMA_LUX13HS = 46,
	
	SENSOR_MODEL_ID_OnSemi_KAI08052 = 47,
	
	SENSOR_MODEL_ID_OnSemi_KAI04070 = 48,
	
	SENSOR_MODEL_ID_OnSemi_KAI16070 = 49,
	
	SENSOR_MODEL_ID_LUXIMA_LUX19HS = 50,
	
	SENSOR_MODEL_ID_LUXIMA_LUX160 = 51,
	
	SENSOR_MODEL_ID_Sony_IMX273 = 52,
	
	SENSOR_MODEL_ID_Sony_IMX287 = 53,
	
	SENSOR_MODEL_ID_OnSemi_KAI29052 = 54,
	
	SENSOR_MODEL_ID_OnSemi_AR1820HS = 55,
	
	SENSOR_MODEL_ID_CMOSIS_COMPIS = 56,
	
	SENSOR_MODEL_ID_GPixel_GSENSE5130 = 57,
	
	SENSOR_MODEL_ID_GPixel_GSENSE400 = 58,
	
	SENSOR_MODEL_ID_GPixel_GSENSE6060_FSI = 59,
	
	SENSOR_MODEL_ID_GPixel_GSENSE6060_BSI = 60,
	
	SENSOR_MODEL_ID_GPixel_GMAX0505 = 61,
	
	SENSOR_MODEL_ID_GPixel_GSENSE2020_BSI = 62,
	
	SENSOR_MODEL_ID_GPixel_GMAX3265 = 63,
	
	SENSOR_MODEL_ID_IMEC_CMV2K_LS100_600_1000 = 17,
	
	SENSOR_MODEL_ID_IMEC_CMV2K_LS150_470_900 = 17,
	
	SENSOR_MODEL_ID_IMEC_CMV2K_SSM4X4_470_620 = 17,
	
	SENSOR_MODEL_ID_IMEC_CMV2K_SSM5X5_600_1000 = 17,
	
	SENSOR_MODEL_ID_IMEC_CMV2K_SSM2X2_RGBNIR = 17,
	
	SENSOR_MODEL_ID_GPixel_GSENSE400_BSI = 58,
	
	SENSOR_MODEL_ID_Sony_IMX411 = 64,
	
	SENSOR_MODEL_ID_IMEC_CMV2K_SSM4X4_REDNIR = 65,
	
	SENSOR_MODEL_ID_Sony_IMX542 = 66,
	
	SENSOR_MODEL_ID_Sony_IMX541 = 67,
	
	SENSOR_MODEL_ID_Sony_IMX540 = 68,
	
	SENSOR_MODEL_ID_Sony_IMX455 = 70,
	
	SENSOR_MODEL_ID_GPixel_GMAX4651 = 69,
	
	SENSOR_MODEL_ID_Sony_IMX530 = 71,
	
	SENSOR_MODEL_ID_Sony_IMX531 = 72,
	
	SENSOR_MODEL_ID_Sony_IMX532 = 73,
	
	SENSOR_MODEL_ID_Sony_IMX461 = 74,
	
	SENSOR_MODEL_ID_Sony_ICX655ALA = 19,
	
	SENSOR_MODEL_ID_Sony_IMX304 = 75,
	
	SENSOR_MODEL_ID_GPixel_GLT5009BSI = 76,
	
	SENSOR_MODEL_ID_Sony_IMX487 = 77,
	
	SENSOR_MODEL_ID_Sony_IMX420 = 78,
	
	SENSOR_MODEL_ID_Sony_IMX421 = 79,
	
	SENSOR_MODEL_ID_Sony_IMX425 = 80,
	
	SENSOR_MODEL_ID_Sony_IMX426 = 81,
	
	SENSOR_MODEL_ID_GPixel_GSPRINT4521 = 82,
	
	SENSOR_MODEL_ID_GPixel_GSPRINT4502 = 83,
	
	SENSOR_MODEL_ID_OnSemi_AR2020 = 84,
	
	SENSOR_MODEL_ID_Sony_IMX675 = 85,
	
	SENSOR_MODEL_ID_Sony_IMX568 = 86,
	
	SENSOR_MODEL_ID_Sony_IMX556 = 87,
	
	SENSOR_MODEL_ID_Sony_IMX990 = 88,
	
	SENSOR_MODEL_ID_Sony_IMX545 = 89,
	
	SENSOR_MODEL_ID_Sony_IMX546 = 90,
	
	SENSOR_MODEL_ID_Sony_IMX547 = 91,
	
	SENSOR_MODEL_ID_Sony_IMX535 = 92,
	
	SENSOR_MODEL_ID_Sony_IMX536 = 93,
	
	SENSOR_MODEL_ID_Sony_IMX537 = 94,
	
	SENSOR_MODEL_ID_Sony_IMX811 = 95,
	
	SENSOR_MODEL_ID_Sony_IMX661 = 96,
	
	SENSOR_MODEL_ID_Teledyne_e2v_Hydra3Dp = 97,
	
	SENSOR_MODEL_ID_GPixel_GCINE4349 = 98,
	
	SENSOR_MODEL_ID_GPixel_GSPRINT4510 = 99,
	
	SENSOR_MODEL_ID_GPixel_GMAX0505_Red_Fox = 61,
	
	SENSOR_MODEL_ID_Sony_IMX250_Polarized = 40,
	
}E_SENSOR_MODEL; 

#ifdef __cplusplus
}
#endif

