
//-------------------------------------------------------------------------------------------------------------------
// xiApi parameters - generated by xiapi_parameters.xsl
// {feature name, feature define name, type, acess mode, custom inreger},

// Basic

					{XI_PRM_EXPOSURE,"XI_PRM_EXPOSURE",xiTypeFloat,"RW",0}, 
					

					{XI_PRM_EXPOSURE_TIME_SELECTOR,"XI_PRM_EXPOSURE_TIME_SELECTOR",xiTypeEnum,"RW",0}, 
					

					{XI_PRM_EXPOSURE_BURST_COUNT,"XI_PRM_EXPOSURE_BURST_COUNT",xiTypeInteger,"RW",0}, 
					

					{XI_PRM_GAIN_SELECTOR,"XI_PRM_GAIN_SELECTOR",xiTypeEnum,"RW",0}, 
					

					{XI_PRM_GAIN,"XI_PRM_GAIN",xiTypeFloat,"RW",0}, 
					

					{XI_PRM_DOWNSAMPLING,"XI_PRM_DOWNSAMPLING",xiTypeEnum,"RW",0}, 
					

					{XI_PRM_DOWNSAMPLING_TYPE,"XI_PRM_DOWNSAMPLING_TYPE",xiTypeEnum,"RW",0}, 
					

					{XI_PRM_TEST_PATTERN_GENERATOR_SELECTOR,"XI_PRM_TEST_PATTERN_GENERATOR_SELECTOR",xiTypeEnum,"RW",0}, 
					

					{XI_PRM_TEST_PATTERN,"XI_PRM_TEST_PATTERN",xiTypeEnum,"RW",0}, 
					

					{XI_PRM_IMAGE_DATA_FORMAT,"XI_PRM_IMAGE_DATA_FORMAT",xiTypeEnum,"RW",0}, 
					

					{XI_PRM_IMAGE_DATA_SIGN,"XI_PRM_IMAGE_DATA_SIGN",xiTypeEnum,"RO",0}, 
					

					{XI_PRM_SHUTTER_TYPE,"XI_PRM_SHUTTER_TYPE",xiTypeEnum,"RW",0}, 
					

					{XI_PRM_SENSOR_TAPS,"XI_PRM_SENSOR_TAPS",xiTypeEnum,"RW",0}, 
					

					{XI_PRM_AEAG,"XI_PRM_AEAG",xiTypeBoolean,"RW",0}, 
					

					{XI_PRM_AEAG_ROI_OFFSET_X,"XI_PRM_AEAG_ROI_OFFSET_X",xiTypeInteger,"RW",0}, 
					

					{XI_PRM_AEAG_ROI_OFFSET_Y,"XI_PRM_AEAG_ROI_OFFSET_Y",xiTypeInteger,"RW",0}, 
					

					{XI_PRM_AEAG_ROI_WIDTH,"XI_PRM_AEAG_ROI_WIDTH",xiTypeInteger,"RW",0}, 
					

					{XI_PRM_AEAG_ROI_HEIGHT,"XI_PRM_AEAG_ROI_HEIGHT",xiTypeInteger,"RW",0}, 
					

					{XI_PRM_SENS_DEFECTS_CORR_LIST_SELECTOR,"XI_PRM_SENS_DEFECTS_CORR_LIST_SELECTOR",xiTypeEnum,"RW",0}, 
					

					{XI_PRM_SENS_DEFECTS_CORR_LIST_CONTENT,"XI_PRM_SENS_DEFECTS_CORR_LIST_CONTENT",xiTypeString,"RW",0}, 
					

					{XI_PRM_SENS_DEFECTS_CORR,"XI_PRM_SENS_DEFECTS_CORR",xiTypeBoolean,"RW",0}, 
					

					{XI_PRM_AUTO_WB,"XI_PRM_AUTO_WB",xiTypeBoolean,"RW",0}, 
					

					{XI_PRM_MANUAL_WB,"XI_PRM_MANUAL_WB",xiTypeCommand,"WO",0}, 
					

					{XI_PRM_WB_ROI_OFFSET_X,"XI_PRM_WB_ROI_OFFSET_X",xiTypeInteger,"RW",0}, 
					

					{XI_PRM_WB_ROI_OFFSET_Y,"XI_PRM_WB_ROI_OFFSET_Y",xiTypeInteger,"RW",0}, 
					

					{XI_PRM_WB_ROI_WIDTH,"XI_PRM_WB_ROI_WIDTH",xiTypeInteger,"RW",0}, 
					

					{XI_PRM_WB_ROI_HEIGHT,"XI_PRM_WB_ROI_HEIGHT",xiTypeInteger,"RW",0}, 
					

					{XI_PRM_WB_KR,"XI_PRM_WB_KR",xiTypeFloat,"RW",0}, 
					

					{XI_PRM_WB_KG,"XI_PRM_WB_KG",xiTypeFloat,"RW",0}, 
					

					{XI_PRM_WB_KB,"XI_PRM_WB_KB",xiTypeFloat,"RW",0}, 
					

					{XI_PRM_WIDTH,"XI_PRM_WIDTH",xiTypeInteger,"RW",0}, 
					

					{XI_PRM_HEIGHT,"XI_PRM_HEIGHT",xiTypeInteger,"RW",0}, 
					

					{XI_PRM_OFFSET_X,"XI_PRM_OFFSET_X",xiTypeInteger,"RW",0}, 
					

					{XI_PRM_OFFSET_Y,"XI_PRM_OFFSET_Y",xiTypeInteger,"RW",0}, 
					

					{XI_PRM_REGION_SELECTOR,"XI_PRM_REGION_SELECTOR",xiTypeInteger,"RW",0}, 
					

					{XI_PRM_REGION_MODE,"XI_PRM_REGION_MODE",xiTypeInteger,"RW",0}, 
					

					{XI_PRM_HORIZONTAL_FLIP,"XI_PRM_HORIZONTAL_FLIP",xiTypeBoolean,"RW",0}, 
					

					{XI_PRM_VERTICAL_FLIP,"XI_PRM_VERTICAL_FLIP",xiTypeBoolean,"RW",0}, 
					

					{XI_PRM_INTERLINE_EXPOSURE_MODE,"XI_PRM_INTERLINE_EXPOSURE_MODE",xiTypeEnum,"RW",0}, 
					

					{XI_PRM_FFC,"XI_PRM_FFC",xiTypeBoolean,"RW",0}, 
					

					{XI_PRM_FFC_FLAT_FIELD_FILE_NAME,"XI_PRM_FFC_FLAT_FIELD_FILE_NAME",xiTypeString,"RW",0}, 
					

					{XI_PRM_FFC_DARK_FIELD_FILE_NAME,"XI_PRM_FFC_DARK_FIELD_FILE_NAME",xiTypeString,"RW",0}, 
					

					{XI_PRM_TOF_READOUT_MODE,"XI_PRM_TOF_READOUT_MODE",xiTypeEnum,"RW",0}, 
					

					{XI_PRM_TOF_MODULATION_FREQUENCY,"XI_PRM_TOF_MODULATION_FREQUENCY",xiTypeFloat,"RW",0}, 
					

					{XI_PRM_TOF_MULTIPLE_PHASES_IN_BUFFER,"XI_PRM_TOF_MULTIPLE_PHASES_IN_BUFFER",xiTypeInteger,"RO",0}, 
					

					{XI_PRM_TOF_PHASES_COUNT,"XI_PRM_TOF_PHASES_COUNT",xiTypeInteger,"RW",0}, 
					

					{XI_PRM_TOF_PHASE_ANGLE,"XI_PRM_TOF_PHASE_ANGLE",xiTypeFloat,"RW",0}, 
					

					{XI_PRM_TOF_PHASE_EXPOSURE_TIME,"XI_PRM_TOF_PHASE_EXPOSURE_TIME",xiTypeFloat,"RW",0}, 
					

					{XI_PRM_TOF_PHASE_SELECTOR,"XI_PRM_TOF_PHASE_SELECTOR",xiTypeInteger,"RW",0}, 
					

// Image Format

					{XI_PRM_BINNING_SELECTOR,"XI_PRM_BINNING_SELECTOR",xiTypeEnum,"RW",0}, 
					

					{XI_PRM_BINNING_VERTICAL_MODE,"XI_PRM_BINNING_VERTICAL_MODE",xiTypeEnum,"RW",0}, 
					

					{XI_PRM_BINNING_VERTICAL,"XI_PRM_BINNING_VERTICAL",xiTypeInteger,"RW",0}, 
					

					{XI_PRM_BINNING_VERTICAL_FLOAT,"XI_PRM_BINNING_VERTICAL_FLOAT",xiTypeFloat,"RW",0}, 
					

					{XI_PRM_BINNING_HORIZONTAL_MODE,"XI_PRM_BINNING_HORIZONTAL_MODE",xiTypeEnum,"RW",0}, 
					

					{XI_PRM_BINNING_HORIZONTAL,"XI_PRM_BINNING_HORIZONTAL",xiTypeInteger,"RW",0}, 
					

					{XI_PRM_BINNING_HORIZONTAL_FLOAT,"XI_PRM_BINNING_HORIZONTAL_FLOAT",xiTypeFloat,"RW",0}, 
					

					{XI_PRM_BINNING_HORIZONTAL_PATTERN,"XI_PRM_BINNING_HORIZONTAL_PATTERN",xiTypeEnum,"RW",0}, 
					

					{XI_PRM_BINNING_VERTICAL_PATTERN,"XI_PRM_BINNING_VERTICAL_PATTERN",xiTypeEnum,"RW",0}, 
					

					{XI_PRM_DECIMATION_SELECTOR,"XI_PRM_DECIMATION_SELECTOR",xiTypeEnum,"RW",0}, 
					

					{XI_PRM_DECIMATION_VERTICAL,"XI_PRM_DECIMATION_VERTICAL",xiTypeInteger,"RW",0}, 
					

					{XI_PRM_DECIMATION_HORIZONTAL,"XI_PRM_DECIMATION_HORIZONTAL",xiTypeInteger,"RW",0}, 
					

					{XI_PRM_DECIMATION_HORIZONTAL_PATTERN,"XI_PRM_DECIMATION_HORIZONTAL_PATTERN",xiTypeEnum,"RW",0}, 
					

					{XI_PRM_DECIMATION_VERTICAL_PATTERN,"XI_PRM_DECIMATION_VERTICAL_PATTERN",xiTypeEnum,"RW",0}, 
					

// AE Setup

					{XI_PRM_EXP_PRIORITY,"XI_PRM_EXP_PRIORITY",xiTypeFloat,"RW",0}, 
					

					{XI_PRM_AG_MAX_LIMIT,"XI_PRM_AG_MAX_LIMIT",xiTypeFloat,"RW",0}, 
					

					{XI_PRM_AE_MAX_LIMIT,"XI_PRM_AE_MAX_LIMIT",xiTypeInteger,"RW",0}, 
					

					{XI_PRM_AEAG_LEVEL,"XI_PRM_AEAG_LEVEL",xiTypeInteger,"RW",0}, 
					

					{XI_PRM_AEAG_SKIP_FRAMES_COUNT,"XI_PRM_AEAG_SKIP_FRAMES_COUNT",xiTypeInteger,"RW",0}, 
					

// Performance

					{XI_PRM_LIMIT_BANDWIDTH,"XI_PRM_LIMIT_BANDWIDTH",xiTypeInteger,"RW",0}, 
					

					{XI_PRM_LIMIT_BANDWIDTH_MODE,"XI_PRM_LIMIT_BANDWIDTH_MODE",xiTypeEnum,"RW",0}, 
					

					{XI_PRM_SENSOR_DATA_BIT_DEPTH,"XI_PRM_SENSOR_DATA_BIT_DEPTH",xiTypeEnum,"RW",0}, 
					

					{XI_PRM_OUTPUT_DATA_BIT_DEPTH,"XI_PRM_OUTPUT_DATA_BIT_DEPTH",xiTypeEnum,"RW",0}, 
					

					{XI_PRM_IMAGE_DATA_BIT_DEPTH,"XI_PRM_IMAGE_DATA_BIT_DEPTH",xiTypeEnum,"RW",0}, 
					

					{XI_PRM_OUTPUT_DATA_PACKING,"XI_PRM_OUTPUT_DATA_PACKING",xiTypeBoolean,"RW",0}, 
					

					{XI_PRM_OUTPUT_DATA_PACKING_TYPE,"XI_PRM_OUTPUT_DATA_PACKING_TYPE",xiTypeEnum,"RW",0}, 
					

// Temperature

					{XI_PRM_IS_COOLED,"XI_PRM_IS_COOLED",xiTypeBoolean,"RO",0}, 
					

					{XI_PRM_COOLING,"XI_PRM_COOLING",xiTypeEnum,"RW",0}, 
					

					{XI_PRM_TARGET_TEMP,"XI_PRM_TARGET_TEMP",xiTypeFloat,"RW",0}, 
					

					{XI_PRM_TEMP_SELECTOR,"XI_PRM_TEMP_SELECTOR",xiTypeEnum,"RW",0}, 
					

					{XI_PRM_TEMP,"XI_PRM_TEMP",xiTypeFloat,"RO",0}, 
					

					{XI_PRM_TEMP_CONTROL_MODE,"XI_PRM_TEMP_CONTROL_MODE",xiTypeEnum,"RW",0}, 
					

					{XI_PRM_CHIP_TEMP,"XI_PRM_CHIP_TEMP",xiTypeFloat,"RO",0}, 
					

					{XI_PRM_HOUS_TEMP,"XI_PRM_HOUS_TEMP",xiTypeFloat,"RO",0}, 
					

					{XI_PRM_HOUS_BACK_SIDE_TEMP,"XI_PRM_HOUS_BACK_SIDE_TEMP",xiTypeFloat,"RO",0}, 
					

					{XI_PRM_SENSOR_BOARD_TEMP,"XI_PRM_SENSOR_BOARD_TEMP",xiTypeFloat,"RO",0}, 
					

					{XI_PRM_TEMP_ELEMENT_SEL,"XI_PRM_TEMP_ELEMENT_SEL",xiTypeEnum,"RW",0}, 
					

					{XI_PRM_TEMP_ELEMENT_VALUE,"XI_PRM_TEMP_ELEMENT_VALUE",xiTypeFloat,"RW",0}, 
					

// Color Correction

					{XI_PRM_CMS,"XI_PRM_CMS",xiTypeEnum,"RW",0}, 
					

					{XI_PRM_CMS_INTENT,"XI_PRM_CMS_INTENT",xiTypeEnum,"RW",0}, 
					

					{XI_PRM_APPLY_CMS,"XI_PRM_APPLY_CMS",xiTypeBoolean,"RW",0}, 
					

					{XI_PRM_INPUT_CMS_PROFILE,"XI_PRM_INPUT_CMS_PROFILE",xiTypeString,"RW",0}, 
					

					{XI_PRM_OUTPUT_CMS_PROFILE,"XI_PRM_OUTPUT_CMS_PROFILE",xiTypeString,"RW",0}, 
					

					{XI_PRM_IMAGE_IS_COLOR,"XI_PRM_IMAGE_IS_COLOR",xiTypeBoolean,"RO",0}, 
					

					{XI_PRM_COLOR_FILTER_ARRAY,"XI_PRM_COLOR_FILTER_ARRAY",xiTypeEnum,"RO",0}, 
					

					{XI_PRM_GAMMAY,"XI_PRM_GAMMAY",xiTypeFloat,"RW",0}, 
					

					{XI_PRM_GAMMAC,"XI_PRM_GAMMAC",xiTypeFloat,"RW",0}, 
					

					{XI_PRM_SHARPNESS,"XI_PRM_SHARPNESS",xiTypeFloat,"RW",0}, 
					

					{XI_PRM_CC_MATRIX_00,"XI_PRM_CC_MATRIX_00",xiTypeFloat,"RW",0}, 
					

					{XI_PRM_CC_MATRIX_01,"XI_PRM_CC_MATRIX_01",xiTypeFloat,"RW",0}, 
					

					{XI_PRM_CC_MATRIX_02,"XI_PRM_CC_MATRIX_02",xiTypeFloat,"RW",0}, 
					

					{XI_PRM_CC_MATRIX_03,"XI_PRM_CC_MATRIX_03",xiTypeFloat,"RW",0}, 
					

					{XI_PRM_CC_MATRIX_10,"XI_PRM_CC_MATRIX_10",xiTypeFloat,"RW",0}, 
					

					{XI_PRM_CC_MATRIX_11,"XI_PRM_CC_MATRIX_11",xiTypeFloat,"RW",0}, 
					

					{XI_PRM_CC_MATRIX_12,"XI_PRM_CC_MATRIX_12",xiTypeFloat,"RW",0}, 
					

					{XI_PRM_CC_MATRIX_13,"XI_PRM_CC_MATRIX_13",xiTypeFloat,"RW",0}, 
					

					{XI_PRM_CC_MATRIX_20,"XI_PRM_CC_MATRIX_20",xiTypeFloat,"RW",0}, 
					

					{XI_PRM_CC_MATRIX_21,"XI_PRM_CC_MATRIX_21",xiTypeFloat,"RW",0}, 
					

					{XI_PRM_CC_MATRIX_22,"XI_PRM_CC_MATRIX_22",xiTypeFloat,"RW",0}, 
					

					{XI_PRM_CC_MATRIX_23,"XI_PRM_CC_MATRIX_23",xiTypeFloat,"RW",0}, 
					

					{XI_PRM_CC_MATRIX_30,"XI_PRM_CC_MATRIX_30",xiTypeFloat,"RW",0}, 
					

					{XI_PRM_CC_MATRIX_31,"XI_PRM_CC_MATRIX_31",xiTypeFloat,"RW",0}, 
					

					{XI_PRM_CC_MATRIX_32,"XI_PRM_CC_MATRIX_32",xiTypeFloat,"RW",0}, 
					

					{XI_PRM_CC_MATRIX_33,"XI_PRM_CC_MATRIX_33",xiTypeFloat,"RW",0}, 
					

					{XI_PRM_DEFAULT_CC_MATRIX,"XI_PRM_DEFAULT_CC_MATRIX",xiTypeCommand,"WO",0}, 
					

					{XI_PRM_CC_MATRIX_NORM,"XI_PRM_CC_MATRIX_NORM",xiTypeBoolean,"RW",0}, 
					

// Device IO

					{XI_PRM_TRG_SOURCE,"XI_PRM_TRG_SOURCE",xiTypeEnum,"RW",0}, 
					

					{XI_PRM_TRG_SOFTWARE,"XI_PRM_TRG_SOFTWARE",xiTypeCommand,"WO",0}, 
					

					{XI_PRM_TRG_SELECTOR,"XI_PRM_TRG_SELECTOR",xiTypeEnum,"RW",0}, 
					

					{XI_PRM_TRG_OVERLAP,"XI_PRM_TRG_OVERLAP",xiTypeEnum,"RW",0}, 
					

					{XI_PRM_ACQ_FRAME_BURST_COUNT,"XI_PRM_ACQ_FRAME_BURST_COUNT",xiTypeInteger,"RW",0}, 
					

					{XI_PRM_TIMESTAMP,"XI_PRM_TIMESTAMP",xiTypeInteger64,"RO",0}, 
					

// GPIO Setup

					{XI_PRM_GPI_SELECTOR,"XI_PRM_GPI_SELECTOR",xiTypeEnum,"RW",0}, 
					

					{XI_PRM_GPI_MODE,"XI_PRM_GPI_MODE",xiTypeEnum,"RW",0}, 
					

					{XI_PRM_GPI_LEVEL,"XI_PRM_GPI_LEVEL",xiTypeInteger,"RO",0}, 
					

					{XI_PRM_GPI_LEVEL_AT_IMAGE_EXP_START,"XI_PRM_GPI_LEVEL_AT_IMAGE_EXP_START",xiTypeInteger,"RO",0}, 
					

					{XI_PRM_GPI_LEVEL_AT_IMAGE_EXP_END,"XI_PRM_GPI_LEVEL_AT_IMAGE_EXP_END",xiTypeInteger,"RO",0}, 
					

					{XI_PRM_GPI_LEVEL_AT_IMAGE_READOUT_END,"XI_PRM_GPI_LEVEL_AT_IMAGE_READOUT_END",xiTypeInteger,"RO",0}, 
					

					{XI_PRM_GPO_SELECTOR,"XI_PRM_GPO_SELECTOR",xiTypeEnum,"RW",0}, 
					

					{XI_PRM_GPO_MODE,"XI_PRM_GPO_MODE",xiTypeEnum,"RW",0}, 
					

					{XI_PRM_METADATA_SAMPLING_MODE,"XI_PRM_METADATA_SAMPLING_MODE",xiTypeEnum,"RW",0}, 
					

					{XI_PRM_LED_SELECTOR,"XI_PRM_LED_SELECTOR",xiTypeEnum,"RW",0}, 
					

					{XI_PRM_LED_MODE,"XI_PRM_LED_MODE",xiTypeEnum,"RW",0}, 
					

					{XI_PRM_DEBOUNCE_EN,"XI_PRM_DEBOUNCE_EN",xiTypeBoolean,"RW",0}, 
					

// Debounce Setup

					{XI_PRM_DEBOUNCE_T0,"XI_PRM_DEBOUNCE_T0",xiTypeInteger,"RW",0}, 
					

					{XI_PRM_DEBOUNCE_T1,"XI_PRM_DEBOUNCE_T1",xiTypeInteger,"RW",0}, 
					

					{XI_PRM_DEBOUNCE_POL,"XI_PRM_DEBOUNCE_POL",xiTypeInteger,"RW",0}, 
					

// Lens Control

					{XI_PRM_LENS_MODE,"XI_PRM_LENS_MODE",xiTypeBoolean,"RW",0}, 
					

					{XI_PRM_LENS_APERTURE_VALUE,"XI_PRM_LENS_APERTURE_VALUE",xiTypeFloat,"RW",0}, 
					

					{XI_PRM_LENS_APERTURE_INDEX,"XI_PRM_LENS_APERTURE_INDEX",xiTypeInteger,"RW",0}, 
					

					{XI_PRM_LENS_FOCUS_MOVEMENT_VALUE,"XI_PRM_LENS_FOCUS_MOVEMENT_VALUE",xiTypeInteger,"RW",0}, 
					

					{XI_PRM_LENS_FOCUS_MOVE,"XI_PRM_LENS_FOCUS_MOVE",xiTypeCommand,"WO",0}, 
					

					{XI_PRM_LENS_FOCAL_LENGTH,"XI_PRM_LENS_FOCAL_LENGTH",xiTypeFloat,"RO",0}, 
					

					{XI_PRM_LENS_FEATURE_SELECTOR,"XI_PRM_LENS_FEATURE_SELECTOR",xiTypeEnum,"RW",0}, 
					

					{XI_PRM_LENS_FEATURE,"XI_PRM_LENS_FEATURE",xiTypeFloat,"RW",0}, 
					

// Device info parameters

					{XI_PRM_DEVICE_NAME,"XI_PRM_DEVICE_NAME",xiTypeString,"RO",0}, 
					

					{XI_PRM_DEVICE_TYPE,"XI_PRM_DEVICE_TYPE",xiTypeString,"RO",0}, 
					

					{XI_PRM_DEVICE_MODEL_ID,"XI_PRM_DEVICE_MODEL_ID",xiTypeInteger,"RO",0}, 
					

					{XI_PRM_SENSOR_MODEL_ID,"XI_PRM_SENSOR_MODEL_ID",xiTypeInteger,"RO",0}, 
					

					{XI_PRM_DEVICE_SN,"XI_PRM_DEVICE_SN",xiTypeString,"RO",0}, 
					

					{XI_PRM_DEVICE_SENS_SN,"XI_PRM_DEVICE_SENS_SN",xiTypeString,"RO",0}, 
					

					{XI_PRM_DEVICE_INSTANCE_PATH,"XI_PRM_DEVICE_INSTANCE_PATH",xiTypeString,"RO",0}, 
					

					{XI_PRM_DEVICE_LOCATION_PATH,"XI_PRM_DEVICE_LOCATION_PATH",xiTypeString,"RO",0}, 
					

					{XI_PRM_DEVICE_USER_ID,"XI_PRM_DEVICE_USER_ID",xiTypeString,"RO",0}, 
					

					{XI_PRM_DEVICE_MANIFEST,"XI_PRM_DEVICE_MANIFEST",xiTypeString,"RO",0}, 
					

					{XI_PRM_IMAGE_USER_DATA,"XI_PRM_IMAGE_USER_DATA",xiTypeInteger,"RW",0}, 
					

// Device acquisition settings

					{XI_PRM_IMAGE_DATA_FORMAT_RGB32_ALPHA,"XI_PRM_IMAGE_DATA_FORMAT_RGB32_ALPHA",xiTypeInteger,"RW",0}, 
					

					{XI_PRM_IMAGE_PAYLOAD_SIZE,"XI_PRM_IMAGE_PAYLOAD_SIZE",xiTypeInteger,"RO",0}, 
					

					{XI_PRM_TRANSPORT_PIXEL_FORMAT,"XI_PRM_TRANSPORT_PIXEL_FORMAT",xiTypeEnum,"RW",0}, 
					

					{XI_PRM_TRANSPORT_DATA_TARGET,"XI_PRM_TRANSPORT_DATA_TARGET",xiTypeEnum,"RW",0}, 
					

					{XI_PRM_SENSOR_CLOCK_FREQ_HZ,"XI_PRM_SENSOR_CLOCK_FREQ_HZ",xiTypeFloat,"RW",0}, 
					

					{XI_PRM_SENSOR_CLOCK_FREQ_INDEX,"XI_PRM_SENSOR_CLOCK_FREQ_INDEX",xiTypeInteger,"RW",0}, 
					

					{XI_PRM_SENSOR_OUTPUT_CHANNEL_COUNT,"XI_PRM_SENSOR_OUTPUT_CHANNEL_COUNT",xiTypeEnum,"RW",0}, 
					

					{XI_PRM_FRAMERATE,"XI_PRM_FRAMERATE",xiTypeFloat,"RW",0}, 
					

					{XI_PRM_COUNTER_SELECTOR,"XI_PRM_COUNTER_SELECTOR",xiTypeEnum,"RW",0}, 
					

					{XI_PRM_COUNTER_VALUE,"XI_PRM_COUNTER_VALUE",xiTypeInteger,"RO",0}, 
					

					{XI_PRM_ACQ_TIMING_MODE,"XI_PRM_ACQ_TIMING_MODE",xiTypeEnum,"RW",0}, 
					

					{XI_PRM_AVAILABLE_BANDWIDTH,"XI_PRM_AVAILABLE_BANDWIDTH",xiTypeInteger,"RO",0}, 
					

					{XI_PRM_BUFFER_POLICY,"XI_PRM_BUFFER_POLICY",xiTypeEnum,"RW",0}, 
					

					{XI_PRM_LUT_EN,"XI_PRM_LUT_EN",xiTypeBoolean,"RW",0}, 
					

					{XI_PRM_LUT_INDEX,"XI_PRM_LUT_INDEX",xiTypeInteger,"RW",0}, 
					

					{XI_PRM_LUT_VALUE,"XI_PRM_LUT_VALUE",xiTypeInteger,"RW",0}, 
					

					{XI_PRM_TRG_DELAY,"XI_PRM_TRG_DELAY",xiTypeInteger,"RW",0}, 
					

					{XI_PRM_TS_RST_MODE,"XI_PRM_TS_RST_MODE",xiTypeEnum,"RW",0}, 
					

					{XI_PRM_TS_RST_SOURCE,"XI_PRM_TS_RST_SOURCE",xiTypeEnum,"RW",0}, 
					

// Extended Device parameters

					{XI_PRM_IS_DEVICE_EXIST,"XI_PRM_IS_DEVICE_EXIST",xiTypeBoolean,"RO",0}, 
					

					{XI_PRM_ACQ_BUFFER_SIZE,"XI_PRM_ACQ_BUFFER_SIZE",xiTypeInteger,"RW",0}, 
					

					{XI_PRM_ACQ_BUFFER_SIZE_UNIT,"XI_PRM_ACQ_BUFFER_SIZE_UNIT",xiTypeInteger,"RW",0}, 
					

					{XI_PRM_ACQ_TRANSPORT_BUFFER_SIZE,"XI_PRM_ACQ_TRANSPORT_BUFFER_SIZE",xiTypeInteger,"RW",0}, 
					

					{XI_PRM_ACQ_TRANSPORT_PACKET_SIZE,"XI_PRM_ACQ_TRANSPORT_PACKET_SIZE",xiTypeInteger,"RW",0}, 
					

					{XI_PRM_BUFFERS_QUEUE_SIZE,"XI_PRM_BUFFERS_QUEUE_SIZE",xiTypeInteger,"RW",0}, 
					

					{XI_PRM_ACQ_TRANSPORT_BUFFER_COMMIT,"XI_PRM_ACQ_TRANSPORT_BUFFER_COMMIT",xiTypeInteger,"RW",0}, 
					

					{XI_PRM_RECENT_FRAME,"XI_PRM_RECENT_FRAME",xiTypeBoolean,"RW",0}, 
					

					{XI_PRM_DEVICE_RESET,"XI_PRM_DEVICE_RESET",xiTypeCommand,"WO",0}, 
					

					{XI_PRM_CONCAT_IMG_MODE,"XI_PRM_CONCAT_IMG_MODE",xiTypeBoolean,"RW",0}, 
					

					{XI_PRM_CONCAT_IMG_COUNT,"XI_PRM_CONCAT_IMG_COUNT",xiTypeInteger,"RW",0}, 
					

					{XI_PRM_CONCAT_IMG_TRANSPORT_IMG_OFFSET,"XI_PRM_CONCAT_IMG_TRANSPORT_IMG_OFFSET",xiTypeInteger,"RO",0}, 
					

					{XI_PRM_PROBE_SELECTOR,"XI_PRM_PROBE_SELECTOR",xiTypeEnum,"RW",0}, 
					

					{XI_PRM_PROBE_VALUE,"XI_PRM_PROBE_VALUE",xiTypeFloat,"RO",0}, 
					

// Sensor Defects Correction

					{XI_PRM_COLUMN_FPN_CORRECTION,"XI_PRM_COLUMN_FPN_CORRECTION",xiTypeEnum,"RW",0}, 
					

					{XI_PRM_ROW_FPN_CORRECTION,"XI_PRM_ROW_FPN_CORRECTION",xiTypeEnum,"RW",0}, 
					

					{XI_PRM_COLUMN_BLACK_OFFSET_CORRECTION,"XI_PRM_COLUMN_BLACK_OFFSET_CORRECTION",xiTypeEnum,"RW",0}, 
					

					{XI_PRM_ROW_BLACK_OFFSET_CORRECTION,"XI_PRM_ROW_BLACK_OFFSET_CORRECTION",xiTypeEnum,"RW",0}, 
					

// Sensor features

					{XI_PRM_SENSOR_MODE,"XI_PRM_SENSOR_MODE",xiTypeEnum,"RW",0}, 
					

					{XI_PRM_HDR,"XI_PRM_HDR",xiTypeBoolean,"RW",0}, 
					

					{XI_PRM_HDR_KNEEPOINT_COUNT,"XI_PRM_HDR_KNEEPOINT_COUNT",xiTypeInteger,"RW",0}, 
					

					{XI_PRM_HDR_T1,"XI_PRM_HDR_T1",xiTypeInteger,"RW",0}, 
					

					{XI_PRM_HDR_T2,"XI_PRM_HDR_T2",xiTypeInteger,"RW",0}, 
					

					{XI_PRM_KNEEPOINT1,"XI_PRM_KNEEPOINT1",xiTypeInteger,"RW",0}, 
					

					{XI_PRM_KNEEPOINT2,"XI_PRM_KNEEPOINT2",xiTypeInteger,"RW",0}, 
					

					{XI_PRM_IMAGE_BLACK_LEVEL,"XI_PRM_IMAGE_BLACK_LEVEL",xiTypeInteger,"RO",0}, 
					

					{XI_PRM_IMAGE_AREA,"XI_PRM_IMAGE_AREA",xiTypeEnum,"RW",0}, 
					

					{XI_PRM_DUAL_ADC_MODE,"XI_PRM_DUAL_ADC_MODE",xiTypeEnum,"RW",0}, 
					

					{XI_PRM_DUAL_ADC_GAIN_RATIO,"XI_PRM_DUAL_ADC_GAIN_RATIO",xiTypeFloat,"RW",0}, 
					

					{XI_PRM_DUAL_ADC_THRESHOLD,"XI_PRM_DUAL_ADC_THRESHOLD",xiTypeInteger,"RW",0}, 
					

					{XI_PRM_COMPRESSION_REGION_SELECTOR,"XI_PRM_COMPRESSION_REGION_SELECTOR",xiTypeInteger,"RW",0}, 
					

					{XI_PRM_COMPRESSION_REGION_START,"XI_PRM_COMPRESSION_REGION_START",xiTypeFloat,"RW",0}, 
					

					{XI_PRM_COMPRESSION_REGION_GAIN,"XI_PRM_COMPRESSION_REGION_GAIN",xiTypeFloat,"RW",0}, 
					

// Version info

					{XI_PRM_VERSION_SELECTOR,"XI_PRM_VERSION_SELECTOR",xiTypeEnum,"RW",0}, 
					

					{XI_PRM_VERSION,"XI_PRM_VERSION",xiTypeString,"RO",0}, 
					

					{XI_PRM_API_VERSION,"XI_PRM_API_VERSION",xiTypeString,"RO",0}, 
					

					{XI_PRM_DRV_VERSION,"XI_PRM_DRV_VERSION",xiTypeString,"RO",0}, 
					

					{XI_PRM_MCU1_VERSION,"XI_PRM_MCU1_VERSION",xiTypeString,"RO",0}, 
					

					{XI_PRM_MCU2_VERSION,"XI_PRM_MCU2_VERSION",xiTypeString,"RO",0}, 
					

					{XI_PRM_MCU3_VERSION,"XI_PRM_MCU3_VERSION",xiTypeString,"RO",0}, 
					

					{XI_PRM_FPGA1_VERSION,"XI_PRM_FPGA1_VERSION",xiTypeString,"RO",0}, 
					

					{XI_PRM_XMLMAN_VERSION,"XI_PRM_XMLMAN_VERSION",xiTypeString,"RO",0}, 
					

					{XI_PRM_HW_REVISION,"XI_PRM_HW_REVISION",xiTypeString,"RO",0}, 
					

					{XI_PRM_FACTORY_SET_VERSION,"XI_PRM_FACTORY_SET_VERSION",xiTypeString,"RO",0}, 
					

// API features

					{XI_PRM_DEBUG_LEVEL,"XI_PRM_DEBUG_LEVEL",xiTypeEnum,"RW",0}, 
					

					{XI_PRM_AUTO_BANDWIDTH_CALCULATION,"XI_PRM_AUTO_BANDWIDTH_CALCULATION",xiTypeBoolean,"RW",0}, 
					

					{XI_PRM_NEW_PROCESS_CHAIN_ENABLE,"XI_PRM_NEW_PROCESS_CHAIN_ENABLE",xiTypeBoolean,"RW",0}, 
					

					{XI_PRM_PROC_NUM_THREADS,"XI_PRM_PROC_NUM_THREADS",xiTypeInteger,"RW",0}, 
					

// Camera FFS

					{XI_PRM_READ_FILE_FFS,"XI_PRM_READ_FILE_FFS",xiTypeString,"RO",0}, 
					

					{XI_PRM_WRITE_FILE_FFS,"XI_PRM_WRITE_FILE_FFS",xiTypeString,"RW",0}, 
					

					{XI_PRM_FFS_FILE_NAME,"XI_PRM_FFS_FILE_NAME",xiTypeString,"RW",0}, 
					

					{XI_PRM_FFS_FILE_ID,"XI_PRM_FFS_FILE_ID",xiTypeInteger,"RO",0}, 
					

					{XI_PRM_FFS_FILE_SIZE,"XI_PRM_FFS_FILE_SIZE",xiTypeInteger,"RO",0}, 
					

					{XI_PRM_FREE_FFS_SIZE,"XI_PRM_FREE_FFS_SIZE",xiTypeInteger64,"RO",0}, 
					

					{XI_PRM_USED_FFS_SIZE,"XI_PRM_USED_FFS_SIZE",xiTypeInteger64,"RO",0}, 
					

					{XI_PRM_FFS_ACCESS_KEY,"XI_PRM_FFS_ACCESS_KEY",xiTypeInteger,"RW",0}, 
					

// APIContextControl

					{XI_PRM_API_CONTEXT_LIST,"XI_PRM_API_CONTEXT_LIST",xiTypeString,"RW",0}, 
					

// Sensor Control

					{XI_PRM_SENSOR_FEATURE_SELECTOR,"XI_PRM_SENSOR_FEATURE_SELECTOR",xiTypeEnum,"RW",0}, 
					

					{XI_PRM_SENSOR_FEATURE_VALUE,"XI_PRM_SENSOR_FEATURE_VALUE",xiTypeInteger,"RW",0}, 
					

// Extended Features

					{XI_PRM_ACQUISITION_STATUS_SELECTOR,"XI_PRM_ACQUISITION_STATUS_SELECTOR",xiTypeEnum,"RW",0}, 
					

					{XI_PRM_ACQUISITION_STATUS,"XI_PRM_ACQUISITION_STATUS",xiTypeEnum,"RO",0}, 
					

					{XI_PRM_DP_UNIT_SELECTOR,"XI_PRM_DP_UNIT_SELECTOR",xiTypeEnum,"RW",0}, 
					

					{XI_PRM_DP_PROC_SELECTOR,"XI_PRM_DP_PROC_SELECTOR",xiTypeEnum,"RW",0}, 
					

					{XI_PRM_DP_PARAM_SELECTOR,"XI_PRM_DP_PARAM_SELECTOR",xiTypeEnum,"RW",0}, 
					

					{XI_PRM_DP_PARAM_VALUE,"XI_PRM_DP_PARAM_VALUE",xiTypeFloat,"RW",0}, 
					

					{XI_PRM_GENTL_DATASTREAM_ENABLED,"XI_PRM_GENTL_DATASTREAM_ENABLED",xiTypeBoolean,"RW",0}, 
					

					{XI_PRM_GENTL_DATASTREAM_CONTEXT,"XI_PRM_GENTL_DATASTREAM_CONTEXT",xiTypeString,"RO",0}, 
					

// User Set Control

					{XI_PRM_USER_SET_SELECTOR,"XI_PRM_USER_SET_SELECTOR",xiTypeEnum,"RW",0}, 
					

					{XI_PRM_USER_SET_LOAD,"XI_PRM_USER_SET_LOAD",xiTypeCommand,"WO",0}, 
					

					{XI_PRM_USER_SET_DEFAULT,"XI_PRM_USER_SET_DEFAULT",xiTypeEnum,"RW",0}, 
					
