'''
ximea

xiapi - python wrapper of c xiApi functions
xidefs - definitions from xiApi

Module provides means to connect to ximea cameras, set and get different
parameters, acquire images, retrieve image data, etc.
'''

__version__ = '4.32.00'
